@@ -3,7 +3,7 @@
 /*
  *  GThumb
  *
- *  Copyright (C) 2001 The Free Software Foundation, Inc.
+ *  Copyright (C) 2001, 2003 The Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
  *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
  */
 
+#include <config.h>
 #include <unistd.h>
 #include <string.h>
 #include <errno.h>
@@ -54,6 +55,8 @@ struct _ImageLoaderPrivateData {
 	gboolean              done;
 	gboolean              error;
 	gboolean              loader_done;
+	gboolean              interrupted;
+	gboolean              loading;
 
 	GTimer               *timer;
 	int                   priority;
@@ -75,13 +78,16 @@ struct _ImageLoaderPrivateData {
 	gboolean              start_loading;
 	GMutex               *start_loading_mutex;
 	GCond                *start_loading_cond;
+
+	LoaderFunc            loader;
+	gpointer              loader_data;
 };
 
 
 enum {
-	ERROR,
-	DONE,
-	PROGRESS,
+	IMAGE_ERROR,
+	IMAGE_DONE,
+	IMAGE_PROGRESS,
 	LAST_SIGNAL
 };
 
@@ -99,17 +105,19 @@ image_loader_finalize__step2 (GObject *object)
         il = IMAGE_LOADER (object);
 	priv = il->priv;
 
-	if (priv->pixbuf)
+	g_mutex_lock (priv->yes_or_no);
+	if (priv->pixbuf != NULL)
 		g_object_unref (G_OBJECT (priv->pixbuf));
 	
-	if (priv->animation)
+	if (priv->animation != NULL)
 		g_object_unref (G_OBJECT (priv->animation));
 
-	if (priv->uri) {
+	if (priv->uri != NULL) {
 		gnome_vfs_uri_unref (priv->uri);
 		priv->uri = NULL;
 	}
-	
+	g_mutex_unlock (priv->yes_or_no);
+
 	g_timer_destroy (priv->timer);
 
 	g_mutex_lock (priv->exit_thread_mutex);
@@ -168,29 +176,29 @@ image_loader_class_init (ImageLoaderClass *class)
 
 	parent_class = g_type_class_peek_parent (class);
 
-	image_loader_signals[ERROR] =
-		g_signal_new ("error",
+	image_loader_signals[IMAGE_ERROR] =
+		g_signal_new ("image_error",
 			      G_TYPE_FROM_CLASS (class),
 			      G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (ImageLoaderClass, error),
+			      G_STRUCT_OFFSET (ImageLoaderClass, image_error),
 			      NULL, NULL,
 			      gthumb_marshal_VOID__VOID,
 			      G_TYPE_NONE, 
 			      0);
-	image_loader_signals[DONE] =
-		g_signal_new ("done",
+	image_loader_signals[IMAGE_DONE] =
+		g_signal_new ("image_done",
 			      G_TYPE_FROM_CLASS (class),
 			      G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (ImageLoaderClass, done),
+			      G_STRUCT_OFFSET (ImageLoaderClass, image_done),
 			      NULL, NULL,
 			      gthumb_marshal_VOID__VOID,
 			      G_TYPE_NONE, 
 			      0);
-	image_loader_signals[PROGRESS] =
-		g_signal_new ("progress",
+	image_loader_signals[IMAGE_PROGRESS] =
+		g_signal_new ("image_progress",
 			      G_TYPE_FROM_CLASS (class),
 			      G_SIGNAL_RUN_FIRST,
-			      G_STRUCT_OFFSET (ImageLoaderClass, progress),
+			      G_STRUCT_OFFSET (ImageLoaderClass, image_progress),
 			      NULL, NULL,
 			      gthumb_marshal_VOID__FLOAT,
 			      G_TYPE_NONE, 
@@ -198,9 +206,9 @@ image_loader_class_init (ImageLoaderClass *class)
 
 	object_class->finalize = image_loader_finalize;
 
-	class->error = NULL;
-	class->done = NULL;
-	class->progress = NULL;
+	class->image_error = NULL;
+	class->image_done = NULL;
+	class->image_progress = NULL;
 }
 
 static void * load_image_thread (void *thread_data);
@@ -210,7 +218,7 @@ image_loader_init (ImageLoader *il)
 {
 	ImageLoaderPrivateData *priv;
 
-	il->priv = g_new (ImageLoaderPrivateData, 1);
+	il->priv = g_new0 (ImageLoaderPrivateData, 1);
 	priv = il->priv;
 
 	priv->pixbuf = NULL;
@@ -225,6 +233,7 @@ image_loader_init (ImageLoader *il)
 	priv->done = FALSE;
 	priv->error = FALSE;
 	priv->loader_done = FALSE;
+	priv->interrupted = FALSE;
 
 	priv->timer = g_timer_new ();
 	priv->priority = GNOME_VFS_PRIORITY_DEFAULT;
@@ -243,9 +252,10 @@ image_loader_init (ImageLoader *il)
 	priv->start_loading_mutex = g_mutex_new ();
 	priv->start_loading_cond = g_cond_new ();
 
-	priv->thread = g_thread_create (load_image_thread, il, TRUE, NULL);
+	priv->loader = NULL;
+	priv->loader_data = NULL;
 
-	/*g_thread_create (load_image_thread, il, FALSE, NULL); FIXME*/
+	priv->thread = g_thread_create (load_image_thread, il, TRUE, NULL);
 }
 
 
@@ -295,16 +305,26 @@ image_loader_new (const gchar *path,
 
 
 void
-image_loader_set_path (ImageLoader *il,
-		       const gchar *path)
+image_loader_set_loader (ImageLoader *il,
+			 LoaderFunc   loader,
+			 gpointer     loader_data)
 {
-	ImageLoaderPrivateData *priv;
-	char                   *escaped_path;
-
 	g_return_if_fail (il != NULL);
 
-	if (path == NULL)
-		return;
+	g_mutex_lock (il->priv->yes_or_no);
+	il->priv->loader = loader;
+	il->priv->loader_data = loader_data;
+	g_mutex_unlock (il->priv->yes_or_no);
+}
+
+
+void
+image_loader_set_path (ImageLoader *il,
+		       const char  *path)
+{
+	ImageLoaderPrivateData *priv;
+
+	g_return_if_fail (il != NULL);
 
 	priv = il->priv;
 
@@ -314,9 +334,13 @@ image_loader_set_path (ImageLoader *il,
 		gnome_vfs_uri_unref (priv->uri);
 		priv->uri = NULL;
 	}
-	escaped_path = gnome_vfs_escape_path_string (path);
-	priv->uri = gnome_vfs_uri_new (escaped_path);
-	g_free (escaped_path);
+
+	if (path != NULL) {
+		char *escaped_path;
+		escaped_path = gnome_vfs_escape_path_string (path);
+		priv->uri = gnome_vfs_uri_new (escaped_path);
+		g_free (escaped_path);
+	}
 
 	g_mutex_unlock (priv->yes_or_no);
 }
@@ -329,7 +353,6 @@ image_loader_set_uri (ImageLoader       *il,
 	ImageLoaderPrivateData *priv;
 
 	g_return_if_fail (il != NULL);
-	g_return_if_fail (uri != NULL);
 	
 	priv = il->priv;
 
@@ -387,16 +410,18 @@ image_loader_sync_pixbuf (ImageLoader *il)
 
 	pixbuf = gdk_pixbuf_animation_get_static_image (priv->animation);
 
-	g_mutex_unlock (priv->yes_or_no);
-
-	if (priv->pixbuf == pixbuf)
+	if (priv->pixbuf == pixbuf) {
+		g_mutex_unlock (priv->yes_or_no);
 		return;
-		
-	if (pixbuf != NULL) 
-		g_object_ref (pixbuf);
+	}
+
 	if (priv->pixbuf != NULL) 
 		g_object_unref (priv->pixbuf);
+	if (pixbuf != NULL) 
+		g_object_ref (pixbuf);
      	priv->pixbuf = pixbuf;
+
+	g_mutex_unlock (priv->yes_or_no);
 }
 
 
@@ -427,18 +452,36 @@ image_loader_sync_pixbuf_from_loader (ImageLoader     *il,
 			priv->animation = NULL;
 	}
 
-	g_mutex_unlock (priv->yes_or_no);
-
 	pixbuf = gdk_pixbuf_loader_get_pixbuf (pb_loader);
 
-	if (priv->pixbuf == pixbuf)
+	if (priv->pixbuf == pixbuf) {
+		g_mutex_unlock (priv->yes_or_no);
 		return;
-		
-	if (pixbuf != NULL) 
-		g_object_ref (pixbuf);
+	}
+
 	if (priv->pixbuf != NULL) 
 		g_object_unref (priv->pixbuf);
+	if (pixbuf != NULL) 
+		g_object_ref (pixbuf);
      	priv->pixbuf = pixbuf;
+
+	g_mutex_unlock (priv->yes_or_no);
+}
+
+
+static void 
+image_loader_interrupted (ImageLoader *il)
+{
+	ImageLoaderPrivateData *priv = il->priv;
+
+	g_mutex_lock (priv->yes_or_no);
+	priv->error = TRUE;
+	g_mutex_unlock (priv->yes_or_no);
+
+	image_loader_stop_common (il, 
+				  priv->done_func, 
+				  priv->done_func_data, 
+				  TRUE);
 }
 
 
@@ -494,12 +537,16 @@ load_image_thread (void *thread_data)
 			break;
 
 		path = image_loader_get_path (il);
-		if (path != NULL)
-			animation = gdk_pixbuf_animation_new_from_file (path, &error);
-		else 
-			animation = NULL;
 
 		g_mutex_lock (priv->yes_or_no);
+
+		if (path != NULL) {
+			if (priv->loader != NULL)
+				animation = (*priv->loader) (path, &error, priv->loader_data);
+			else
+				animation = gdk_pixbuf_animation_new_from_file (path, &error);
+		} else 
+			animation = NULL;
 		
 		priv->loader_done = TRUE;
 
@@ -510,12 +557,13 @@ load_image_thread (void *thread_data)
 		if ((animation == NULL) || (error != NULL)) {
 			priv->error = TRUE;
 			priv->done = FALSE;
-			g_clear_error (&error);
+			if (error != NULL)
+				g_clear_error (&error);
 		} else {
 			priv->error = FALSE;
 			priv->done = TRUE;
 		}
-		
+
 		g_mutex_unlock (priv->yes_or_no);
 
 		g_free (path);
@@ -536,27 +584,32 @@ image_loader_stop__final_step (ImageLoader *il)
 	error = priv->error;
 	priv->done = TRUE;
 	g_mutex_unlock (priv->yes_or_no);
-	
-	if (! error) 
+
+	if (! error && ! priv->interrupted && priv->loading) 
 		image_loader_sync_pixbuf (il);
+	priv->loading = FALSE;
 	
 	priv->done_func = NULL;
 	if (done_func != NULL)
 		(*done_func) (priv->done_func_data);
 	
-	if (! priv->emit_signal)
+	if (! priv->emit_signal || priv->interrupted) {
+		priv->interrupted = FALSE;
 		return;
+	}
 		
+	priv->interrupted = FALSE;
+
 	if (error)
 		g_signal_emit (G_OBJECT (il), 
-			       image_loader_signals[ERROR], 0);
+			       image_loader_signals[IMAGE_ERROR], 0);
 	else
 		g_signal_emit (G_OBJECT (il), 
-			       image_loader_signals[DONE], 0);
+			       image_loader_signals[IMAGE_DONE], 0);
 }
 
 
-static gint
+static int
 check_thread (gpointer data)
 {
 	ImageLoader            *il = data;
@@ -578,7 +631,10 @@ check_thread (gpointer data)
 
 	/**/
 
-	if (loader_done && done) 
+	if (loader_done && priv->interrupted)
+		image_loader_interrupted (il);
+
+	else if (loader_done && done) 
 		image_loader_done (il);
 	
 	else if (loader_done && error) 
@@ -641,18 +697,21 @@ image_loader_start__step2 (ImageLoader *il)
 	g_return_if_fail (il != NULL);
 
 	priv = il->priv;
+
+	g_mutex_lock (priv->yes_or_no);
+
 	g_return_if_fail (priv->uri != NULL);
 
 	g_timer_reset (priv->timer);
 	g_timer_start (priv->timer);
 
-	g_mutex_lock (priv->yes_or_no);
 	priv->done = FALSE;
 	priv->error = FALSE;
 	priv->loader_done = FALSE;
+	priv->loading = TRUE;
+	uri_list = g_list_prepend (NULL, gnome_vfs_uri_dup (priv->uri));
 	g_mutex_unlock (priv->yes_or_no);
 
-	uri_list = g_list_prepend (NULL, priv->uri);
 	gnome_vfs_async_get_file_info (& (priv->info_handle),
 				       uri_list,
 				       (GNOME_VFS_FILE_INFO_DEFAULT
@@ -660,7 +719,12 @@ image_loader_start__step2 (ImageLoader *il)
 				       GNOME_VFS_PRIORITY_DEFAULT,
 				       get_file_info_cb,
 				       il);
-	g_list_free (uri_list);
+	{
+		GList *scan;
+		for (scan = uri_list; scan; scan = scan->next) 
+			gnome_vfs_uri_unref((GnomeVFSURI*) scan->data);
+		g_list_free (uri_list);
+	}
 }
 
 
@@ -670,8 +734,15 @@ image_loader_start (ImageLoader *il)
 	ImageLoaderPrivateData *priv;
 
 	g_return_if_fail (il != NULL);
+
 	priv = il->priv;
-	g_return_if_fail (priv->uri != NULL);
+
+	g_mutex_lock (priv->yes_or_no);
+	if (priv->uri == NULL) {
+		g_mutex_unlock (priv->yes_or_no);		
+		return;
+	}
+	g_mutex_unlock (priv->yes_or_no);
 
 	image_loader_stop_common (il, (DoneFunc) image_loader_start__step2, il, FALSE);
 }
@@ -728,7 +799,7 @@ image_loader_stop (ImageLoader *il,
 		   gpointer     done_func_data)
 {
 	ImageLoaderPrivateData *priv;
-	gboolean                emit_sig;
+	/*gboolean                emit_sig;*/
 
 	g_return_if_fail (il != NULL);
 
@@ -738,10 +809,19 @@ image_loader_stop (ImageLoader *il,
 	priv->error = FALSE;
 	g_mutex_unlock (priv->yes_or_no);
 
-	/* emit a signal only if there is an operation to stop */
-	emit_sig = (priv->info_handle != NULL);
+	if (priv->loading) {
+		priv->emit_signal = TRUE;
+		priv->interrupted = TRUE;
+		priv->done_func = done_func;
+		priv->done_func_data = done_func_data;
+	} else
+		image_loader_stop_common (il, done_func, done_func_data, FALSE);
 
-	image_loader_stop_common (il, done_func, done_func_data, emit_sig);
+	/* FIXME: emit a signal only if there is an operation to stop */
+	/*
+	  emit_sig = (priv->info_handle != NULL);
+	  image_loader_stop_common (il, done_func, done_func_data, emit_sig);
+	*/
 }
 
 
@@ -767,12 +847,8 @@ image_loader_stop_with_error (ImageLoader *il,
 GdkPixbuf *
 image_loader_get_pixbuf (ImageLoader *il)
 {
-	ImageLoaderPrivateData *priv;
-
 	g_return_val_if_fail (il != NULL, NULL);
-	priv = il->priv;
-
-	return priv->pixbuf;
+	return il->priv->pixbuf;
 }
 
 
@@ -851,20 +927,21 @@ image_loader_get_path (ImageLoader *il)
 	priv = il->priv;
 
 	g_mutex_lock (priv->yes_or_no);
-	uri = priv->uri;
-	if (uri != NULL)
-		gnome_vfs_uri_ref (uri);
-	g_mutex_unlock (priv->yes_or_no);
 
-	if (uri == NULL) 
+	if (priv->uri == NULL) {
+		g_mutex_unlock (priv->yes_or_no);
                 return NULL;
+	}
 
-        path = gnome_vfs_uri_to_string (uri,
-					GNOME_VFS_URI_HIDE_TOPLEVEL_METHOD);
+	uri = gnome_vfs_uri_dup (priv->uri);
+
+        path = gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_TOPLEVEL_METHOD);
         esc_path = gnome_vfs_unescape_string (path, NULL);
         g_free (path);
 	gnome_vfs_uri_unref (uri);
 
+	g_mutex_unlock (priv->yes_or_no);
+
         return esc_path;
 }
 
@@ -873,15 +950,14 @@ GnomeVFSURI *
 image_loader_get_uri (ImageLoader *il)
 {
 	ImageLoaderPrivateData *priv;
-	GnomeVFSURI            *uri;
+	GnomeVFSURI            *uri = NULL;
 
 	g_return_val_if_fail (il != NULL, 0);
 	priv = il->priv;
 
 	g_mutex_lock (priv->yes_or_no);
-	uri = priv->uri;
-	if (uri != NULL)
-		gnome_vfs_uri_ref (uri);
+	if (priv->uri != NULL)
+		uri = gnome_vfs_uri_dup (priv->uri);
 	g_mutex_unlock (priv->yes_or_no);
 
 	return uri;
@@ -891,12 +967,8 @@ image_loader_get_uri (ImageLoader *il)
 GTimer *
 image_loader_get_timer (ImageLoader *il)
 {
-	ImageLoaderPrivateData *priv;
-
 	g_return_val_if_fail (il != NULL, 0);
-	priv = il->priv;
-
-	return priv->timer;
+	return il->priv->timer;
 }
 
 
@@ -915,9 +987,9 @@ image_loader_load_from_pixbuf_loader (ImageLoader *il,
 	g_mutex_unlock (il->priv->yes_or_no);
 
 	if (error)
-		g_signal_emit (G_OBJECT (il), image_loader_signals[ERROR], 0);
+		g_signal_emit (G_OBJECT (il), image_loader_signals[IMAGE_ERROR], 0);
 	else
-		g_signal_emit (G_OBJECT (il), image_loader_signals[DONE], 0);
+		g_signal_emit (G_OBJECT (il), image_loader_signals[IMAGE_DONE], 0);
 }
 
 
@@ -939,10 +1011,7 @@ image_loader_load_from_image_loader (ImageLoader *to,
 	}
 
 	if (from->priv->uri != NULL)
-		to->priv->uri = gnome_vfs_uri_ref (from->priv->uri);
-
-	g_mutex_unlock (to->priv->yes_or_no);
-	g_mutex_unlock (from->priv->yes_or_no);
+		to->priv->uri = gnome_vfs_uri_dup (from->priv->uri);
 
 	/**/
 
@@ -958,9 +1027,6 @@ image_loader_load_from_image_loader (ImageLoader *to,
 
 	/**/
 
-	g_mutex_lock (to->priv->yes_or_no);
-	g_mutex_lock (from->priv->yes_or_no);
-
 	if (to->priv->animation) {
 		g_object_unref (to->priv->animation);
 		to->priv->animation = NULL;
@@ -977,8 +1043,8 @@ image_loader_load_from_image_loader (ImageLoader *to,
 	g_mutex_unlock (from->priv->yes_or_no);
 
 	if (error)
-		g_signal_emit (G_OBJECT (to), image_loader_signals[ERROR], 0);
+		g_signal_emit (G_OBJECT (to), image_loader_signals[IMAGE_ERROR], 0);
 	else
-		g_signal_emit (G_OBJECT (to), image_loader_signals[DONE], 0);
+		g_signal_emit (G_OBJECT (to), image_loader_signals[IMAGE_DONE], 0);
 }
 
