@@ -43,6 +43,10 @@
 #include "preferences.h"
 #include "icons/pixbufs.h"
 #include "typedefs.h"
+#include "comments.h"
+#include "pixbuf-utils.h"
+#include "commands-impl.h"
+
 
 #define ICON_NAME_DIRECTORY "gnome-fs-directory"
 
@@ -51,14 +55,14 @@ GList          *window_list = NULL;
 FullScreen     *fullscreen;
 char          **file_urls, **dir_urls;
 int             n_file_urls, n_dir_urls;
-int             StartInFullscreen;
-int             StartSlideshow;
+int             StartInFullscreen = FALSE;
+int             StartSlideshow = FALSE;
 int             ViewFirstImage = FALSE;
 int             HideSidebar = FALSE;
 gboolean        ExitAll = FALSE;
 char           *ImageToDisplay = NULL;
 gboolean        FirstStart = TRUE;
-
+gboolean        ImportPhotos = FALSE;
 
 static gboolean        view_comline_catalog = FALSE;
 static gboolean        view_single_image = FALSE;
@@ -77,14 +81,18 @@ static gboolean load_session        (void);
 
 
 struct poptOption options[] = {
-	{ "fullscreen", 'f', POPT_ARG_NONE, &StartInFullscreen, 0,
+	{ "fullscreen", '\0', POPT_ARG_NONE, &StartInFullscreen, 0,
 	  N_("Start in fullscreen mode"),
 	  0 },
 
-	{ "slideshow", 's', POPT_ARG_NONE, &StartSlideshow, 0,
+	{ "slideshow", '\0', POPT_ARG_NONE, &StartSlideshow, 0,
 	  N_("Automatically start a slideshow"),
 	  0 },
 
+	{ "import-photos", '\0', POPT_ARG_NONE, &ImportPhotos, 0,
+	  N_("Automatically import digital camera photos"),
+	  0 },
+
 	{ NULL, '\0', 0, NULL, 0 }
 };
 
@@ -95,14 +103,18 @@ struct poptOption options[] = {
 static gboolean 
 check_whether_to_set_fullscreen (gpointer data) 
 {
-	if (first_window == NULL)
-		return FALSE;
-
-	if (StartInFullscreen) {
-		StartInFullscreen = FALSE;
+	if ((first_window != NULL) && StartInFullscreen) 
 		fullscreen_start (fullscreen, first_window);
-	}
-	
+	StartInFullscreen = FALSE;
+	return FALSE;
+}
+
+
+static gboolean 
+check_whether_to_import_photos (gpointer data) 
+{
+	if ((first_window != NULL) && ImportPhotos) 
+		file_camera_import_command_impl (NULL, first_window, NULL);		
 	return FALSE;
 }
 
@@ -129,25 +141,23 @@ main (int argc, char *argv[])
 				      GNOME_PARAM_APP_LIBDIR, GTHUMB_LIBDIR,
 				      NULL);
 
-	if (! g_thread_supported ())
+	if (! g_thread_supported ()) {
 		g_thread_init (NULL);
-	gdk_threads_init ();
+		gdk_threads_init ();
+	}
 
 	g_object_get_property (G_OBJECT (program),
 			       GNOME_PARAM_POPT_CONTEXT,
 			       g_value_init (&value, G_TYPE_POINTER));
 	pctx = g_value_get_pointer (&value);
-
 	glade_gnome_init ();
-
 	gthumb_init ();
-
 	initialize_data (pctx);
 	poptFreeContext (pctx);
-
 	prepare_app ();
 
 	g_idle_add (check_whether_to_set_fullscreen, NULL);
+	g_idle_add (check_whether_to_import_photos, NULL);
 
 	bonobo_main ();
 
@@ -364,8 +374,11 @@ prepare_app ()
 	if (! view_comline_catalog 
 	    && (n_dir_urls == 0) 
 	    && (n_file_urls == 0)) {
+		if (ImportPhotos)
+			preferences_set_startup_location (NULL);
 		current_window = window_new ();
-		gtk_widget_show (current_window->app);
+		if (!ImportPhotos)
+			gtk_widget_show (current_window->app);
 		if (first_window == NULL)
 			first_window = current_window;
 	}
@@ -774,21 +787,19 @@ get_folder_pixbuf (double icon_size)
 	}
 	
 	if (folder_pixbuf != NULL) {
-		int       new_w, new_h;
-		int       w, h;
-		double    factor;
+		int w, h;
 		
 		w = gdk_pixbuf_get_width (folder_pixbuf);
 		h = gdk_pixbuf_get_height (folder_pixbuf);
-		
-		factor = MIN (icon_size / w, icon_size / h);
-		new_w  = MAX ((gint) (factor * w), 1);
-		new_h  = MAX ((gint) (factor * h), 1);
-				
-		pixbuf = gdk_pixbuf_scale_simple (folder_pixbuf,
-						  new_w,
-						  new_h,
-						  GDK_INTERP_BILINEAR);
+		if (scale_keepping_ratio (&w, &h, icon_size, icon_size)) 
+			pixbuf = gdk_pixbuf_scale_simple (folder_pixbuf,
+							  w,
+							  h,
+							  GDK_INTERP_BILINEAR);
+		else {
+			pixbuf = folder_pixbuf;
+			g_object_ref (pixbuf);
+		}
 	}
 
 	return pixbuf;
@@ -966,12 +977,47 @@ load_session (void)
 
 
 int
-get_default_folder_pixbuf_size (GtkWidget *widget)
+get_folder_pixbuf_size_for_list (GtkWidget *widget)
 {
 	int icon_width, icon_height;
 
 	gtk_icon_size_lookup_for_settings (gtk_widget_get_settings (widget),
-                                           ICON_GTK_SIZE,
+                                           GTK_ICON_SIZE_SMALL_TOOLBAR,
                                            &icon_width, &icon_height);
 	return MAX (icon_width, icon_height);
 }
+
+
+int
+get_folder_pixbuf_size_for_menu (GtkWidget *widget)
+{
+	int icon_width, icon_height;
+
+	gtk_icon_size_lookup_for_settings (gtk_widget_get_settings (widget),
+                                           GTK_ICON_SIZE_MENU,
+                                           &icon_width, &icon_height);
+	return MAX (icon_width, icon_height);
+}
+
+
+gboolean
+folder_is_film (const char *folder)
+{
+	CommentData *cdata;
+	gboolean     film = FALSE;
+
+	folder = pref_util_remove_prefix (folder);
+
+	cdata = comments_load_comment (folder);
+	if (cdata != NULL) {
+		int i;
+		for (i = 0; i < cdata->keywords_n; i++)
+			if (g_utf8_collate (cdata->keywords[i], _("Film")) == 0) {
+				film = TRUE;
+				break;
+			}
+		comment_data_free (cdata);
+	}
+
+	return film;
+}
